import 'dart:collection';

import '../src/bdlive_flutter_viewer_impl.dart';
import 'bdlive_model.dart';


class BdlLiveRoomStatusListener {
  /**
   * Callback when entering the live room page foreground
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onLiveRoomActivityResume;

  /**
   * Callback when entering the live room page background
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onLiveRoomActivityPause;

  /**
   * Callback when entering the live room page destoryed
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onLiveRoomFullDestroyed;

  /**
   * Callback when the floating window of the live room is about to appear
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onFloatViewWillAppear;

  /**
   * Callback when the floating window of the live room is about to close
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  void Function(BdlActivityConfig activity)? onFloatViewWillClose;

  /**
   * Callback when an ad/product card in the live room is clicked, allowing the client to listen to this callback for custom redirection
   * @param url: The redirection link corresponding to the ad/product card
   * @param type: The type of ad/product card, see BdlViewerOpenUrlType
   * @param enableFloating: Whether the user has set to display content as a floating layer in the corporate live broadcast console.
      true: Display content as a floating layer on the viewing page.
      false: Display content on a new page.
   */
  void Function(String url, BdlViewerOpenUrlType type, bool enableFloating)? onOpenUrlCallback;

  BdlLiveRoomStatusListener({
    this.onLiveRoomActivityResume,
    this.onLiveRoomActivityPause,
    this.onLiveRoomFullDestroyed,
    this.onFloatViewWillAppear,
    this.onFloatViewWillClose,
    this.onOpenUrlCallback,
  });
}

abstract class BdliveViewerApi {
  static BdliveViewerApi _instance = BdliveFlutterViewerImpl();

  static BdliveViewerApi get instance => _instance;

  static set instance(BdliveViewerApi instance) {
    _instance = instance;
  }


  /**
   * Enter the complete live room page
   * @param activity: Configuration when entering the live room, see BdlActivityConfig
   */
  Future<bool> enterLiveRoom(BdlActivityConfig activity);

  /**
   * Leave the complete live room page. If the floating window of the live room is currently displayed, it will also be destroyed.
   */
  Future<void> leaveLiveRoom();

  /**
   * Simulate manually clicking the exit live room icon in the complete live room page; subsequent behavior is the same as that of a viewer clicking the exit live room icon.
   */
  Future<void> manualClickLiveRoomExitBtn();

  /**
   * Set a listener for the complete live room page to listen for events of the complete live room page
   * @param listener: Listener for the complete live room page, see BdlLiveRoomStatusListener
   */
  Future<void> setLiveRoomStatusListener(BdlLiveRoomStatusListener? listener);

  /**
   * Used in conjunction with the onOpenUrlCallback callback in BdlLiveRoomStatusListener,
   * clients can call this API to use some default redirection capabilities of the SDK
   * @param url: The redirection link corresponding to the ad/product card
   * @param type: The type of ad/product card, see BdlViewerOpenUrlType
   * @param enableFloating: Whether to allow displaying content as a floating layer on the viewing page.
      true: Display content as a floating layer on the viewing page; a webview pop-up will be shown in the complete live page to display the URL.
      false: Display content on a new page; the app's deep link mechanism will be used to find and open the corresponding page.
   */
  Future<void> openUrl(String url, BdlViewerOpenUrlType type, bool enableFloating);
}
